'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createStore = exports.getActionIds = exports.createActions = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _uniqueid = require('uniqueid');

var _uniqueid2 = _interopRequireDefault(_uniqueid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//export {promiseMiddleware} from './promise'
//export {loggingMiddleware} from './logging'
//export {crashReporter} from './crashReporter'
//export {authMiddleware} from './auth'

//  weak
var stateConstants = ['BEGIN', 'SUCCESS', 'FAILURE'];

// Create actions so we don't need to use constants :)
var createActions = exports.createActions = function createActions(actionObj) {

  var baseId = (0, _uniqueid2.default)();

  //
  // E.g.
  // actionObj = {getSettings: () => { return <Promise|Function|Object> } }
  //

  var actionNameToMethod = _lodash2.default.map(actionObj, function (actionCreator, actionName) {

    var actionId = baseId() + '-' + actionName;

    var method = function method() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var result /*: Promise | Function | Object */ = actionCreator.apply(undefined, args);

      // Promise (async)
      //if (result instanceof Promise) { // This didn't work sometimes due to multiple Promise implementations floating around.
      if ('function' === typeof result.then) {
        return {
          types: stateConstants.map(function (state) {
            return actionId + '-' + state;
          }),
          promise: result
        };
      }

      // Function (async)
      if (typeof result === 'function') {
        return function () {
          return (0, _extends3.default)({
            type: actionId
          }, result.apply(undefined, arguments) || {});
        };
      }

      // Object (sync)
      return (0, _extends3.default)({
        type: actionId
      }, result || {});
    };

    method._id = actionId;

    return [actionName, method];
  });

  return _lodash2.default.fromPairs(actionNameToMethod);
};

// Get action ids from actions created with `createActions`
var getActionIds = exports.getActionIds = function getActionIds(actionCreators) {

  console.log(actionCreators);
  var ids = _lodash2.default.mapValues(actionCreators, function (value, key) {
    return value._id;
  });

  // Add handlers for BEGIN, SUCCESS and FAILURE for promises.
  _lodash2.default.each(ids, function (value, key) {
    _lodash2.default.each(stateConstants, function (state) {
      ids[key + _lodash2.default.capitalize(state.toLowerCase())] = value + '-' + state;
    });
  });

  return ids;
};

// Replace switch statements in reducers (taken from the Redux README).
var createStore = exports.createStore = function createStore(initialState, handlers) {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments[1];
    return handlers[action.type] ? handlers[action.type](state, action) : state;
  };
};
//# sourceMappingURL=index.js.map
