// @flow weak
import _ from 'lodash'
import uniqueId from 'uniqueid'

//export {promiseMiddleware} from './promise'
//export {loggingMiddleware} from './logging'
//export {crashReporter} from './crashReporter'
//export {authMiddleware} from './auth'

const stateConstants = ['BEGIN', 'SUCCESS', 'FAILURE']

// Create actions so we don't need to use constants :)
export const createActions = (actionObj) => {

  const baseId = uniqueId()

  //
  // E.g.
  // actionObj = {getSettings: () => { return <Promise|Function|Object> } }
  //

  const actionNameToMethod:Array<any> = _.map(actionObj, (actionCreator:Function, actionName:string) => {

    const actionId = baseId() + '-' + actionName

    const method:Function = (...args) => {

      const result /*: Promise | Function | Object */ = actionCreator(...args)

      // Promise (async)
      //if (result instanceof Promise) { // This didn't work sometimes due to multiple Promise implementations floating around.
      if ('function' === typeof result.then) {
        return {
          types: stateConstants.map((state) => `${actionId}-${state}`),
          promise: result,
        }
      }

      // Function (async)
      if (typeof result === 'function') {
        return (...args) => {
          return {
            type: actionId,
            ...(result(...args) || {})
          }
        }
      }

      // Object (sync)
      return {
        type: actionId,
        ...(result || {})
      }

    }

    method._id = actionId

    return [actionName, method]

  })

  return _.fromPairs(actionNameToMethod)

}

// Get action ids from actions created with `createActions`
export const getActionIds = (actionCreators) => {

  console.log(actionCreators)
  let ids = _.mapValues(actionCreators, (value, key) => {
    return value._id
  })

  // Add handlers for BEGIN, SUCCESS and FAILURE for promises.
  _.each(ids, (value, key) => {
    _.each(stateConstants, (state) => {
      ids[key + _.capitalize(state.toLowerCase())] = `${value}-${state}`
    })
  })

  return ids

}

// Replace switch statements in reducers (taken from the Redux README).
export const createStore = (initialState, handlers) => {
  return (state = initialState, action) =>
    handlers[action.type] ?
      handlers[action.type](state, action) :
      state
}
